/*******************************************************************************
 * Copyright (C) 1999-2010 Maxim Integrated Products, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL MAXIM INTEGRATED PRODUCTS BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Maxim Integrated Products
 * shall not be used except as stated in the Maxim Integrated Products
 * Branding Policy.
 *
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Maxim Integrated Products retains all ownership rights.
 *
 *     Module Name: Digital Voltmeter
 *     Description: Sample Application for the MAXQ2010 that displays a voltage 
 *      on the LCD based on ADC inputs. The channel displayed is controlled by 
 *      the 5-way switch. Holding down SW3 will halt the ADC.
 *     Filename: voltmeter.c
 *     Author: SH
 *     Created: 06/16/2010
 *
 *******************************************************************************
 */

#include "lcd_2010.h"
#include <iomaxq2010.h>
/*
  Hex codes for decimal numbers   They are split up for easy assignments to LCD# 
  For example, to write a 5 to the first character slot, use
  LCD0 = 0x5a;   or     LCD0 = decimal[ (5*2) ]
  LCD1 = 0x24;          LCD1 = decimal[ (5*2) + 1 ]
  The first byte in the hexcode always goes in an even LCD register.
*/
const int decimal[20] = {0x7E, 0x42, /* 0 */
                          0x06, 0x00, /* 1 */
                          0x3C, 0x24, /* 2 */
                          0x1E, 0x24, /* 3 */
                          0x46, 0x24, /* 4 */
                          0x5A, 0x24, /* 5 */
                          0x7A, 0x24, /* 6 */
                          0x0E, 0x00, /* 7 */
                          0x7E, 0x24, /* 8 */
                          0x4E, 0x24};/* 9 */
/* 
  Hex codes for letters of the alphabet.
  To write C to the second character slot use
  LCD2 = 0x78;    or    LCD2 = alphabet[ 2('c' - 'a') ]
  LCD3 = 0x00;          LCD3 = alphabet[ 2('C' - 'A') + 1 ]
*/
const int alphabet[52] = {0x6E, 0x24, /* A */
                          0x7A, 0xa4, /* B */
                          0x78, 0x00, /* C */
                          0x7E, 0x00, /* D */
                          0x78, 0x04, /* E */
                          0x68, 0x04, /* F */
                          0x7A, 0x20, /* G */
                          0x66, 0x24, /* H */
                          0x18, 0x81, /* I */
                          0x36, 0x00, /* J */
                          0x60, 0x54, /* K */
                          0x70, 0x00, /* L */
                          0x66, 0x48, /* M */
                          0x66, 0x18, /* N */
                          0x7E, 0x00, /* O */
                          0x6C, 0x24, /* P */
                          0x7E, 0x10, /* Q */
                          0x6C, 0x34, /* R */
                          0x5A, 0x24, /* S */
                          0x08, 0x81, /* T */
                          0x76, 0x00, /* U */
                          0x60, 0x42, /* V */
                          0x66, 0x12, /* W */
                          0x00, 0x5A, /* X */
                          0x00, 0x49, /* Y */
                          0x18, 0x42};/* Z */

/*******************************************************************************
 *  void initializeLCD(void)
 * 
 *  Configure the LCD registers and clear the screen.
 *
 * Inputs:
 *   None
 * Outputs:
 *   Clears LCD screen.
 * Returns:
 *   None
 * Notes: This must be called before using any LCD functions.
 */
void initializeLCD(void)
{
   LCRA = 0x1860;   /* 1/4 duty, 32kHz clock, ground VADJ internally
                        000
                           11            - DUTY : 1/4
                             0000        - FRM  : Frame freq
                                 1       - LCCS : 32kHz
                                  1      - LRIG : Ground VADJ
                                   x     -        Reserved
                                    0000 - LRA  : Minimum
                    */                    
   LCFG = 0x7F;     /* Activate segments on P0-P3, enable LCD, normal mode
                        01111    - PCF  : P0/P1/P2/P3 in segment mode
                             1   - SMO  : Enabled when in Stop mode
                              1  - OPM  : Normal operation
                               1 - DPE  : Display enabled
                    */
   clearLCD();
}

/*******************************************************************************
 *  void clearLCD(void)
 * 
 *  Clear the LCD screen.
 *
 */
void clearLCD(void)
{
   LCD0  = 0x00;    /* Clear all display segments */
   LCD1  = 0x00;
   LCD2  = 0x00;
   LCD3  = 0x00;
   LCD4  = 0x00;
   LCD5  = 0x00;
   LCD6  = 0x00;
   LCD7  = 0x00;
   LCD8  = 0x00;
   LCD9  = 0x00;
   LCD10 = 0x00;
   LCD11 = 0x00;
   LCD12 = 0x00;
   LCD13 = 0x00;
   LCD14 = 0x00;
   LCD15 = 0x00;
}

/*******************************************************************************
 *  void writeVoltage(int ADCvalue)
 * 
 *  Converts 0-4096 to 0-3.3000 and writes it on the LCD screen.
 *
 * Inputs:
 *   Value from 0 to 4096
 * Outputs:
 *   #.#### V to the LCD screen
 * Returns:
 *   None
 * Note: Adding the decimal point later solves the issue of integer truncation 
 */
void writeVoltage(int ADCvalue)
{
  int digit1, digit2, digit3, digit4, digit5;
  
  unsigned int voltage = ADCvalue * 3.3 * 10000 / 4095;

  digit1 = voltage / 10000;
  voltage = voltage % 10000;
  digit2 = voltage / 1000;
  voltage = voltage % 1000;
  digit3 = voltage / 100;
  voltage = voltage % 100;
  digit4 = voltage / 10;
  voltage = voltage % 10;
  digit5 = voltage;
  
  /* Digit1 goes in LCD0 and LCD1 with the highbyte in LCD0 */
  /* Digit2 goes in LCD2 and LCD3 with the highbyte in LCD2 and so on */
  LCD0 = decimal[2*digit1] + 0x01;      /* Adds the decimal point */
  LCD1 = decimal[ (2*digit1)+1 ];
  LCD2 = decimal[2*digit2];
  LCD3 = decimal[ (2*digit2)+1 ];
  LCD4 = decimal[2*digit3];
  LCD5 = decimal[ (2*digit3)+1 ];
  LCD6 = decimal[2*digit4];
  LCD7 = decimal[ (2*digit4)+1 ];
  LCD8 = decimal[2*digit5];
  LCD9 = decimal[ (2*digit5)+1 ]; 
  LCD10 = 0x00; /* Space */
  LCD11 = 0x00;
  LCD12 = 0x60; /* Hexcode for V */
  LCD13 = 0x42;
}

/*******************************************************************************
 *  void writeChannel(int ADCchannel)
 * 
 *  Writes Channel# where # is the current ADC channel
 *
 * Inputs:
 *   Value from 0 to 7
 * Outputs:
 *   Channel# to the LCD screen
 * Returns:
 *   None
 */
void writeChannel(int ADCchannel)
{
  LCD0 = 0x78; /* Hexcode for  */
  LCD1 = 0x00; /* C */
  LCD2 = 0x66;
  LCD3 = 0x24; /* H */
  LCD4 = 0x6E;
  LCD5 = 0x24; /* A */
  LCD6 = LCD8 = 0x66;
  LCD7 = LCD9 = 0x18; /* NN */
  LCD10 = 0x78;
  LCD11 = 0x04; /* E */
  LCD12 = 0x70;
  LCD13 = 0x00; /* L */
  LCD14 = decimal[2*ADCchannel];
  LCD15 = decimal[(2*ADCchannel)+1]; /* # */
} 
