/*******************************************************************************
 * Copyright (C) 1999-2010 Maxim Integrated Products, All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL MAXIM INTEGRATED PRODUCTS BE LIABLE FOR ANY CLAIM, DAMAGES
 * OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of Maxim Integrated Products
 * shall not be used except as stated in the Maxim Integrated Products
 * Branding Policy.
 *
 * The mere transfer of this software does not imply any licenses
 * of trade secrets, proprietary technology, copyrights, patents,
 * trademarks, maskwork rights, or any other form of intellectual
 * property whatsoever. Maxim Integrated Products retains all ownership rights.
 *
 *     Module Name: Digital Voltmeter
 *     Description: Sample Application for the MAXQ2010 that displays a voltage 
 *      on the LCD based on ADC inputs. The channel displayed is controlled by 
 *      the 5-way switch. Holding down SW3 will halt the ADC.
 *     Filename: voltmeter.c
 *     Author: SH
 *     Created: 06/16/2010
 *
 *******************************************************************************
 */

#include <iomaxq2010.h>
#include "lcd_2010.h"
#include "voltmeter.h"

#define SWITCHMASK  0x3E

unsigned int values[8][5] = {{0,0,0,0,0},
/* 8x5 matrix containing */  {0,0,0,0,0},
/* the 5 most recent     */  {0,0,0,0,0},
/* samples. Used for     */  {0,0,0,0,0},
/* averaging.            */  {0,0,0,0,0},
                             {0,0,0,0,0},
                             {0,0,0,0,0},
                             {0,0,0,0,0}};
int main(void)
{
   unsigned int ADCvalues[8];
   unsigned int i;
   char switchInput;
   int ADCchannel = 6;
   int hold = 0;
   
   initializeLCD();       /* Configure the LCD registers and clear the screen. */
   PD5 &= ~0x04;  /* Allow holding SW3 to stop the sampling */
   PO5 |= 0x04;   /* Put a weak pullup on Pin 2 of Port 5 */
   
   PD4 = 0xC1;    /* Outputs - P4[7,6,0], Inputs - P4[5:1] */
   PO4 = 0x3E;    /* Drive P4[7,6,0] low, and have weak pullups on P4[5:1] */
   
   ADCN   = 0x0300;       /* ADC clock = sysclock/8  */
   ADST   = 0x0010;       /* Select configuration register 0 */
   ADDATA = 0x00;         /* Single-ended conversion on AN0, ADST autoincrements to config reg 1 */
   ADDATA = 0x01;         /* Single-ended conversion on AN1, ADST autoincrements to config reg 2 */
   ADDATA = 0x02;         /* Single-ended conversion on AN2, ... */
   ADDATA = 0x03;         /* Single-ended conversion on AN3 */
   ADDATA = 0x04;         /* Single-ended conversion on AN4 */
   ADDATA = 0x05;         /* Single-ended conversion on AN5 */
   ADDATA = 0x06;         /* Single-ended conversion on AN6 */
   ADDATA = 0x07;         /* Single-ended conversion on AN7 */
   ADADDR = 0x07;         /* Single conversion sequence (length 8) */

   while (1)
   {
      ADST |= 0x0040;                 /* Start conversion */

      while ((ADST & 0x0040) != 0);   /* Wait for conversion to complete */
      
      ADST   = 0x0000;                /* Select data register 0 */
      for (i=0; i < 8; i++)
      {
        ADCvalues[i] = ADDATA;      /* Get conversion results */
        /* The data register automatically increments when a read ADDATA occurs */
      }
      queue(ADCvalues);             /* Shift out the last values and shift in the new ones */
      
      do
      {
        clearLCD();      
        writeVoltage(average(ADCchannel));  /* Write the current channel value to the screen */
      
        delay_in_ms(35);  /* Delay for the clearLCD. Otherwise the screen gets dim. */
      
        switchInput = PI4 & SWITCHMASK; /* Get the 5-way switch inputs */
        switch (switchInput)
        {
          case (~0x02 & SWITCHMASK):
          case (~0x08 & SWITCHMASK):
            ADCchannel--;               /* Reduce the channel number */
            if (ADCchannel < 0)         /* Make sure the channel is still valid for indexing */
            {
              ADCchannel = 7;
            }
            writeChannel(ADCchannel);   /* Display Channel# */
            delay_in_ms(1000);          /* Allow the message to be seen for 1 second*/
            break;
          case (~0x04 & SWITCHMASK):
          case (~0x10 & SWITCHMASK):
            ADCchannel++;               /* Increment the channel number */
            ADCchannel %= 8;            /* Make sure the channel number is still valid for indexing */
            writeChannel(ADCchannel);   /* Display Channel# */
            delay_in_ms(1000);          /* Allow the message to be seen */
            break;
          case (~0x20 & SWITCHMASK):
            writeChannel(ADCchannel);   /* Display Channel# */
            delay_in_ms(1000);          /* Allow the message to be seen */
            break;                      
          default: break;
        }
        
        switchInput = PI5 & 0x04;   /* Get the pushbutton input */
        if (switchInput == 0)
        {
          hold = 1;
        }
        else
        {
          hold = 0;
        }
      }while (hold == 1); /* Even though SW3 is pressed, halting the ADC, the ADCchannel may be changed to display other frozen channels. */  
   }
}

/*******************************************************************************
 *  unsigned int average(int ADCchannel)
 * 
 *  Average the five samples of the given ADC channel.
 *
 * Inputs:
 *   Value of 0 to 7
 * Outputs:
 *   None
 * Returns:
 *   Value of 0 - 4096
 */
unsigned int average(int ADCchannel) /* Returns the average of the 5 samples for one ADCchannel */
{
  return (values[ADCchannel][0] + 
          values[ADCchannel][1] +
          values[ADCchannel][2] + 
          values[ADCchannel][3] +
          values[ADCchannel][4]) / 5;
}

/*******************************************************************************
 *  void queue(unsigned int* ADCvalues)
 * 
 *  Writes in the most recent samples for all channels by overwriting the oldest samples.
 *
 * Inputs:
 *   Pointer to the array of 8 most recent samples
 * Outputs:
 *   None
 * Returns:
 *   None
 */
void queue(unsigned int* ADCvalues)
{
  static int index = 4;
  int i;
  for (i = 0; i < 8; i++)
  {
    values[i][index] = ADCvalues[i];
  }
  index--;
  if (index < 0)
  { /* Make sure the index is still valid */
    index = 4;
  }
}

/*******************************************************************************
 *  void delay(int milliseconds)
 * 
 *  Delays around 1 millisecond with a 10 Mhz clock.
 *
 * Inputs:
 *   Amount of milliseconds to wait
 * Outputs:
 *   Around one second of waittime.
 * Returns:
 *   None
 * Note: This is not exactly 1 millisecond. Timer interrupts should be used if
 *       an exact wait time is desired.
 */
void delay_in_ms(int milliseconds)  
{
  unsigned int i, j;
  for (j = 0; j < milliseconds; j++)
  {
    for (i = 0; i < 1452; i++);
  }
}
